import { r as registerInstance, h } from './p-597600b0.js';
import { t as toolsInfo, j as imageDirPath } from './p-3ab399ac.js';

const adsMainCss = ":root{font-family:Roboto}.d-none{display:none}.v-hidden{visibility:hidden}.c-pointer{cursor:pointer}.disabled{cursor:not-allowed;opacity:0.6}.fw-bold{font-weight:bold}::-webkit-scrollbar-track{background-color:var(--color-ti-bg-grey);border-radius:5px}::-webkit-scrollbar{background-color:var(--color-ti-bg-grey);height:5px;width:5px}::-webkit-scrollbar-thumb{background-color:var(--color-ti-text-grey);border-radius:5px}*{box-sizing:border-box}:host{display:block;height:100%;width:100%}gc-widget-tab-container{height:calc(100% - 72px);margin:0px;width:100%}gc-widget-tab-panel{height:100%;left:-3px;margin:0;width:100%}.v-separator{border-left:1px solid var(--color-ti-bg-grey);height:70%;margin:auto}";

const AdsMain = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    // ---------- Internal variables ----------
    this.availableTools = [];
    this.listOfTools = [];
    /**
     * To update GC tab container selected page index
     * @param ev
     */
    this.updateSelectedPageIndex = (ev) => {
      this.selectedPageIndex = ev.detail.value;
    };
    this.isShowLoader = false;
    this.selectedDeviceName = "";
    this.selectedPageIndex = 0;
    this.toastMessage = "";
  }
  // ---------- Event handlers ----------
  onChangeSelectedDevice(ev) {
    this.availableTools = ev.detail.availableTools;
    this.selectedDeviceName = ev.detail.deviceName;
    let modelRegister = document.getElementById("gc_model_register");
    {
      this.availableTools.includes("register-map")
        ? (modelRegister.innerHTML = `
    <gc-model-register
    id=${this.selectedDeviceName.replace("-", "_")}
    register-info="dist/config/systeminfo/${this.selectedDeviceName.toLocaleLowerCase()}_regmap_page0.json"
    ></gc-model-register>`)
        : "";
    }
  }
  onSetLoaderState(ev) {
    this.isShowLoader = ev.detail.show;
  }
  onShowToast(ev) {
    this.showToastMessage(ev.detail.message);
  }
  onNavigatePage(ev) {
    this.updateSelectedPageIndex({
      // At 0th index home page is present, incremented by 1
      detail: { value: ev.detail.tabIndex + 1 },
    });
  }
  // ---------- Helper methods ----------
  /**
   * To update toast message and trigger toast
   * @param message
   */
  showToastMessage(message) {
    this.toastMessage = message;
  }
  render() {
    return [
      h("ads-loader", { isShow: this.isShowLoader }),
      h("gc-widget-menubar", { id: "mb", layout: true, horizontal: true, "product-icon-folder-path": "./assets/" }, h("div", null, h("ads-device-selection", { selectedDeviceName: this.selectedDeviceName })), h("div", { class: "v-separator" }), h("gc-widget-menuitem", { id: "mm_file", label: "File", layout: true, vertical: true }, h("gc-widget-menuaction", { id: "ma_exit", label: "Exit", "action-id": "cmd_exit", icon: "round:action:exit_to_app" }), h("gc-widget-menuaction", { id: "ma_copy", label: "Device URL", "action-id": "cmd_copy", icon: "filled:content:content_copy" })), h("gc-widget-menuitem", { id: "mm_help", label: "Help", layout: true, vertical: true }, h("gc-widget-menuaction", { id: "ma_about", label: "About", "action-id": "cmd_open_about_dialog", icon: "round:action:help" }))),
      h("gc-widget-tab-container", { layout: true, horizontal: true, position: "left", index: this.selectedPageIndex, "initial-index": this.selectedPageIndex, "no-labels": true, "on-index-changed": this.updateSelectedPageIndex }, h("gc-widget-tab-panel", { class: "container", label: "Home", "icon-name": "round:action:home", tooltip: "Home", "center-justified": true, layout: true, vertical: true }, h("ads-home", { availableTools: this.availableTools, selectedDeviceName: this.selectedDeviceName })), this.availableTools.map(tool => {
        const Tag = `ads-${tool}`;
        const iconName = tool.startsWith("crc") ? "crc" : tool;
        return (h("gc-widget-tab-panel", { class: "container", label: toolsInfo[tool].name, "icon-name": `main_pageIcons:${iconName}`, iconPath: imageDirPath, tooltip: toolsInfo[tool].name, "center-justified": true, layout: true, vertical: true }, h(Tag, { selectedDevice: this.selectedDeviceName, selectedPageIndex: this.selectedPageIndex, selectedToolName: tool })));
      })),
      h("gc-widget-statusbar", { id: "statusbar", "show-branding-image": true }),
      h("ads-toast", { message: this.toastMessage }),
    ];
  }
};
AdsMain.style = adsMainCss;

export { AdsMain as ads_main };

//# sourceMappingURL=p-a44c875c.entry.js.map