/**
 * The utility APIs in this file are used in the output computation of different deivces
 */

export const decToBinStr = value => {
  return value.toString(2);
};

export const decToBinArrMSBToLSB = (decimal, noOfBits) => {
  return decimal.toString(2).padStart(noOfBits, 0).split("").map(Number);
};

export const decToHexStr = (decimal, noOfBits, isTwosComplement = false) => {
  if (decimal < 0 && isTwosComplement) {
    decimal = (decimal >>> 0).toString(16).slice(-noOfBits / 4);
  }

  return (
    "0x" +
    decimal
      .toString(16)
      .toUpperCase()
      .padStart(noOfBits / 4, 0)
  );
};

export const getDecNum = (value, noOfBits, isTwosComplement = false) => {
  let maxValue = 2 ** parseInt(noOfBits);
  let decimalValue = Math.round(value);

  if (isTwosComplement && decimalValue >= maxValue / 2) {
    decimalValue -= maxValue;
  }

  return decimalValue;
};

export const getMultiplierForUnit = unit => {
  const unitsLookUp = {
    V: 1,
    mV: 1e-3,
    μV: 1e-6,
    nV: 1e-9,
  };

  return unitsLookUp[unit];
};

/**
 * To get table cell object
 * @param cellInfo
 * @returns Table cell object
 */
const getTableCell = cellInfo => {
  return {
    attributes: {
      class: cellInfo.classList || "",
      rowspan: cellInfo.rowspan || 1,
      colspan: cellInfo.colspan || 1,
    },
    value: cellInfo.value === undefined ? "" : cellInfo.value,
  };
};

/**
 * To get tabe row object
 * @param cellList
 * @param type
 * @returns Table row object
 */
export const getTableRow = (cellList, type = "body") => {
  return {
    type,
    entries: cellList.reduce((acc, cellInfo) => {
      acc.push(getTableCell(cellInfo));
      return acc;
    }, []),
  };
};

export const isHexUnit = unit => {
  return unit === "Hex";
};

export const updateHypentoUnderscore = selectedDevice => {
  return selectedDevice.replace("-", "_");
};
